


import java.io.File;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.UUID;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.docx4j.XmlUtils;
import org.docx4j.customXmlProperties.DatastoreItem;
import org.docx4j.jaxb.Context;
import org.docx4j.model.datastorage.BindingHandler;
import org.docx4j.model.datastorage.CustomXmlDataStorage;
import org.docx4j.model.datastorage.CustomXmlDataStorageImpl;
import org.docx4j.openpackaging.exceptions.InvalidFormatException;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.docx4j.openpackaging.parts.CustomXmlDataStoragePart;
import org.docx4j.openpackaging.parts.CustomXmlDataStoragePropertiesPart;
import org.docx4j.openpackaging.parts.Parts;
import org.docx4j.openpackaging.parts.WordprocessingML.MainDocumentPart;
import org.docx4j.wml.SdtPr;
import org.docx4j.wml.Tag;
import org.w3c.dom.Document;
import org.w3c.dom.Element;


public class CustomXmlBindingTest {
	
	public static JAXBContext context = org.docx4j.jaxb.Context.jc; 

	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		
		String inputfilepath = "C:\\Projects\\AVAM\\officeXML\\Test_CC.docx";				
		String outputfilepath = "C:\\Projects\\AVAM\\officeXML\\Test_CC-OUT.docx";
		String outputPdf = "C:\\Projects\\AVAM\\officeXML\\Test_CC-OUT.pdf";
		
		// Load the Package
		WordprocessingMLPackage wordMLPackage;
		wordMLPackage = WordprocessingMLPackage.load(new java.io.File(inputfilepath));
		
		
		CustomXmlDataStoragePart customXmlDataStoragePart = injectCustomXmlDataStoragePart(wordMLPackage.getMainDocumentPart(),
				wordMLPackage.getParts());
		
		String dataBindingStoreItemId = addProperties(customXmlDataStoragePart);	
		
		// Go through all the sdt's and create CTDataBinding
		MainDocumentPart documentPart = wordMLPackage.getMainDocumentPart();		
		String xpathSdt = "//w:sdt";
		List<Object> list = null;
		try {
			list = documentPart.getJAXBNodesViaXPath(xpathSdt, false);
		} catch (JAXBException e) {
			e.printStackTrace();
		}
			
		int i = 0;
		for(Object raw : list) {
			i++;			
			Object o = XmlUtils.unwrap(raw);
			Method m = o.getClass().getDeclaredMethod("getSdtPr");			
			SdtPr sdtPr = (SdtPr)m.invoke(o);			
			Tag tag = sdtPr.getTag();
										
			if (sdtPr.getDataBinding()== null) {									
				if (tag==null) continue;
				String xpath = "/root/" + tag.getVal();		
				org.docx4j.wml.CTDataBinding dataBinding = Context.getWmlObjectFactory().createCTDataBinding();				
				dataBinding.setStoreItemID(dataBindingStoreItemId);
				dataBinding.setXpath(xpath);
				dataBinding.setParent(sdtPr);
				//dataBinding.setPrefixMappings("n0");				
				sdtPr.setDataBinding(dataBinding);					
			}			
		}

		
//		CustomXmlDataStoragePart part = wordMLPackage.getCustomXmlDataStorageParts().get(dataBindingStoreItemId.toLowerCase());			
//		if (part==null) {
//			System.out.println("Couldn't locate part by storeItemId " + dataBindingStoreItemId);
//		}

		// Apply the bindings
		BindingHandler.applyBindings(wordMLPackage.getMainDocumentPart());
		System.out.println(XmlUtils.marshaltoString(wordMLPackage.getMainDocumentPart().getJaxbElement(), true, true));		
		
		// Display as a PDF
		//wordMLPackage.setFontMapper(new IdentityPlusMapper());		
//		org.docx4j.convert.out.pdf.PdfConversion c = new org.docx4j.convert.out.pdf.viaXSLFO.Conversion(wordMLPackage);
//		OutputStream os = new java.io.FileOutputStream(outputPdf);      
//		c.output(os);
	      
		wordMLPackage.save(new java.io.File(outputfilepath) );
         
		System.out.println("..done"); 
	} 
		

	public static CustomXmlDataStoragePart injectCustomXmlDataStoragePart(MainDocumentPart parent, Parts parts) throws Exception {
		
		CustomXmlDataStoragePart customXmlDataStoragePart =	new CustomXmlDataStoragePart(parts);
		
		String[] contentControlsStrings = readContentControls(parent);
		
		CustomXmlDataStorage data = new CustomXmlDataStorageImpl();
		data.setDocument(createCustomXmlDocument(contentControlsStrings));		
		customXmlDataStoragePart.setData(data);						
		parent.addTargetPart(customXmlDataStoragePart);

		return customXmlDataStoragePart;
	}


	public static org.w3c.dom.Document createCustomXmlDocument(String[] contentControlsStrings) {
		
		Document doc = null;		
		HashMap<String, String> mappings = new HashMap<String, String>();
		
		try {
			DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docFactory.newDocumentBuilder();

			// root elements
			doc = docBuilder.newDocument();
			Element rootElement = doc.createElement("root");
			doc.appendChild(rootElement);

			Hashtable<String, String> contentControls = new Hashtable<String, String>();
			for (int count = 0; count < contentControlsStrings.length; count++) {
				if ((count % 2) == 0) {
					contentControls.put(contentControlsStrings[count], "''");	
				} else {
					contentControls.put(contentControlsStrings[count], "CC_" + contentControlsStrings[count]);	
				}										
			}
			
			Enumeration<String> contentControlKey = contentControls.keys();
			while(contentControlKey.hasMoreElements()) {
				String key = contentControlKey.nextElement();
				Element contentControlElement = doc.createElement(key);
				contentControlElement.appendChild(doc.createTextNode(contentControls.get(key)));
				rootElement.appendChild(contentControlElement);
				System.out.println("key = " + key + "; value = " + contentControls.get(key));
			}

			// write the content into xml file
			String outputXmlPath = "C:\\Projects\\AVAM\\officeXML\\CustomXml.xml";

			TransformerFactory transformerFactory = TransformerFactory.newInstance();
			Transformer transformer = transformerFactory.newTransformer();
			DOMSource source = new DOMSource(doc);
			StreamResult result = new StreamResult(new File(outputXmlPath));
			transformer.transform(source, result);

			System.out.println("Done - CustomXml.xml ");

		} catch (ParserConfigurationException pce) {
			pce.printStackTrace();
		} catch (TransformerException tfe) {
			tfe.printStackTrace();
		}
		
		return doc;

		
	}

	public static String[] readContentControls(MainDocumentPart parent) {

		ArrayList<String> fields = new ArrayList<String>();
		
		String xpathSdt = "//w:sdt/w:sdtPr/w:tag";
		List<Object> list = null;
		try {
			list = parent.getJAXBNodesViaXPath(xpathSdt, false);
		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		for (Iterator<Object> it = list.iterator(); it.hasNext();) {
			Object o = XmlUtils.unwrap(it.next());
			if (o instanceof Tag) {
				Tag tag = (Tag) o;
				String val = tag.getVal();				
				fields.add(val);				
				//System.out.println("Tag.Val: " + val);
			}
		}
		
		String[] strArray = new String[fields.size()];
		return fields.toArray(strArray);

	}
	public static String addProperties(
			CustomXmlDataStoragePart customXmlDataStoragePart)
			throws InvalidFormatException {

		CustomXmlDataStoragePropertiesPart part = new CustomXmlDataStoragePropertiesPart();

		org.docx4j.customXmlProperties.ObjectFactory of = new org.docx4j.customXmlProperties.ObjectFactory();

		DatastoreItem dsi = of.createDatastoreItem();
		String newItemId = "{" + UUID.randomUUID().toString() + "}";
		dsi.setItemID(newItemId);

		part.setJaxbElement(dsi);

		customXmlDataStoragePart.addTargetPart(part);
		
		return newItemId;
	}
}
